# tests/test_cleaner.py
import sys
import os
import pytest
from text_cleaner_for_py.cleaner import (
    remove_accents,
    remove_special_characters,
    to_snake_case,
    to_camel_case,
    to_pascal_case,
    remove_html,
    remove_extra_spaces,
    clean_text
)

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))


# 🔡 Teste para remoção de acentos
def test_remove_accents():
    assert remove_accents("Olá, você está bem?") == "Ola, voce esta bem?"

# ✂ Teste para remoção de caracteres especiais
def test_remove_special_characters():
    assert remove_special_characters("Olá! @#$%") == "Olá "

# 🔄 Testes de conversão de cases
def test_to_snake_case():
    assert to_snake_case("Texto Exemplo Aqui!") == "texto_exemplo_aqui"

def test_to_camel_case():
    assert to_camel_case("Texto Exemplo Aqui!") == "textoExemploAqui"

def test_to_pascal_case():
    assert to_pascal_case("Texto Exemplo Aqui!") == "TextoExemploAqui"

# 🌐 Teste para remoção de HTML
def test_remove_html():
    html_text = "<div><h1>Título</h1><p>Parágrafo</p></div>"
    assert remove_html(html_text) == "Título Parágrafo"

# 🔍 Teste para remoção de espaços extras
def test_remove_extra_spaces():
    assert remove_extra_spaces("Texto    com   espaços   extras.") == "Texto com espaços extras."

# 🧹 Testes para limpeza completa do texto
def test_clean_text_lower():
    texto = "<h1>Olá, Mundo!</h1>    Bem-vindo."
    assert clean_text(texto, case="lower") == "ola mundo bemvindo"

def test_clean_text_upper():
    texto = "<h1>Olá, Mundo!</h1>    Bem-vindo."
    assert clean_text(texto, case="upper") == "OLA MUNDO BEMVINDO"

def test_clean_text_title():
    texto = "<h1>Olá, Mundo!</h1>    Bem-vindo."
    assert clean_text(texto, case="title") == "Ola Mundo Bemvindo"

def test_clean_text_snake():
    texto = "<h1>Olá, Mundo!</h1>    Bem-vindo."
    assert clean_text(texto, case="snake") == "ola_mundo_bemvindo"

def test_clean_text_camel():
    texto = "<h1>Olá, Mundo!</h1>    Bem-vindo."
    assert clean_text(texto, case="camel") == "olaMundoBemvindo"

def test_clean_text_pascal():
    texto = "<h1>Olá, Mundo!</h1>    Bem-vindo."
    assert clean_text(texto, case="pascal") == "OlaMundoBemvindo"

# ⚠️ Teste para formato inválido
def test_clean_text_invalid_case():
    with pytest.raises(ValueError, match="⚠️ Formato de case inválido"):
        clean_text("Texto de teste", case="invalido")


import pytest
import sys
import os
from text_cleaner_for_py.cleaner_v1 import (
    normalize_text,
    remove_html_tags,
    clean_whitespace,
    filter_letters,
    filter_numbers,
    remove_stopwords,
)

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

@pytest.fixture
def sample_text():
    return "<p>Olá, mundo! Este é um exemplo de texto com <b>HTML</b> e stopwords.</p>"


def test_normalize_text(sample_text):
    result = normalize_text(sample_text)
    expected = "ola mundo este e um exemplo de texto com html e stopwords"
    assert result == expected


def test_remove_html_tags(sample_text):
    result = remove_html_tags(sample_text)
    expected = "Olá, mundo! Este é um exemplo de texto com HTML e stopwords."
    assert result == expected


def test_clean_whitespace():
    texto = "Texto   com   espaços    e  \n \n quebras."
    result = clean_whitespace(texto)
    expected = "Texto com espaços e quebras."
    assert result == expected


def test_filter_letters():
    texto = "Texto 123 com #caracteres$ %especiais&*"
    result = filter_letters(texto)
    expected = "Texto com caracteres especiais"
    assert result == expected


def test_filter_numbers():
    texto = "Telefone: 123-456-789"
    result = filter_numbers(texto)
    expected = "123456789"
    assert result == expected


def test_remove_stopwords():
    texto = "Este é um texto simples para teste de stopwords."
    result = remove_stopwords(texto, language="portuguese")
    expected = "texto simples teste stopwords."
    assert result == expected


# 🏃 **Execução dos testes**
if __name__ == "__main__":
    pytest.main(["-v", "tests/test_cleaner_v1.py"])



# 📦 text_cleaner/cleaner_v1.py

import re
import unicodedata

import nltk
from bs4 import BeautifulSoup
from nltk.corpus import stopwords

# 📥 Garantir que o corpus de stopwords está disponível
try:
    nltk.data.find("corpora/stopwords")
except LookupError:
    nltk.download("stopwords")


def normalize_text(text: str) -> str:
    """🔡 Converte texto para minúsculas e remove acentos e caracteres especiais (após remover HTML).""" # noqa501
    text = remove_html_tags(text)
    text = text.lower()
    text = unicodedata.normalize('NFKD', text).encode('ASCII', 'ignore').decode('utf-8') # noqa501
    text = re.sub(r'[^a-z0-9\s]', '', text)
    return clean_whitespace(text)


def remove_html_tags(text: str) -> str:
    """🧹 Remove tags HTML do texto."""
    return BeautifulSoup(text, "html.parser").get_text()


def clean_whitespace(text: str) -> str:
    """🧹 Remove espaços em excesso e quebras de linha."""
    return re.sub(r'\s+', ' ', text).strip()


def filter_letters(text: str) -> str:
    """🔍 Mantém apenas letras no texto."""
    text = re.sub(r'[^a-zA-Z\s]', '', text)
    return clean_whitespace(text)


def filter_numbers(text: str) -> str:
    """🔍 Mantém apenas números no texto."""
    return re.sub(r'[^0-9]', '', text)


def remove_stopwords(text: str, language: str = 'portuguese') -> str:
    """🔍 Remove stopwords do texto com base no idioma especificado."""
    stop_words = set(stopwords.words(language))
    words = text.split()
    filtered_words = [word for word in words if word.lower() not in stop_words]
    return ' '.join(filtered_words)


# 🌟 Exemplo de uso
if __name__ == "__main__":
    sample_text = "<p>Olá, mundo! Este é um exemplo de texto com <b>HTML</b> e stopwords.</p>" # noqa501
    print("Original:", sample_text)
    print("Normalizado:", normalize_text(sample_text))
    print("Sem HTML:", remove_html_tags(sample_text))
    print("Espaços limpos:", clean_whitespace(sample_text))
    print("Somente letras:", filter_letters(sample_text))
    print("Somente números:", filter_numbers("Telefone: 123-456-789"))
    print("Sem stopwords:", remove_stopwords(sample_text, language='portuguese')) # noqa501
    print("Sem stopwords:", remove_stopwords(sample_text, language='portuguese')) # noqa501



import re
import unicodedata
from typing import List, Dict, Union
from datetime import datetime
import emoji
import langdetect
from langdetect import DetectorFactory
from nltk.stem import SnowballStemmer, WordNetLemmatizer
import nltk

# Garantir que os recursos necessários do NLTK estão disponíveis
try:
    nltk.data.find('corpora/wordnet')
except LookupError:
    nltk.download('wordnet')

# Configurar o detector de idioma para ser determinístico
DetectorFactory.seed = 0

# Mapeamento de meses em português
MONTHS_PT = {
    1: 'janeiro',
    2: 'fevereiro',
    3: 'março',
    4: 'abril',
    5: 'maio',
    6: 'junho',
    7: 'julho',
    8: 'agosto',
    9: 'setembro',
    10: 'outubro',
    11: 'novembro',
    12: 'dezembro'
}

class AdvancedTextCleaner:
    def __init__(self):
        self.stemmers = {}
        self.lemmatizers = {}
        
    def detect_language(self, text: str) -> str:
        """Detecta o idioma do texto."""
        try:
            return langdetect.detect(text)
        except:
            return 'unknown'

    def remove_emojis(self, text: str) -> str:
        """Remove emojis e emoticons do texto."""
        return emoji.replace_emoji(text, '')

    def remove_urls(self, text: str) -> str:
        """Remove URLs do texto."""
        url_pattern = r'https?://\S+|www\.\S+'
        return re.sub(url_pattern, '', text)

    def remove_emails(self, text: str) -> str:
        """Remove endereços de email do texto."""
        email_pattern = r'\S+@\S+'
        return re.sub(email_pattern, '', text)

    def normalize_numbers(self, text: str) -> str:
        """Normaliza números no texto (ex: 1º -> primeiro)."""
        number_map = {
            '1º': 'primeiro', '2º': 'segundo', '3º': 'terceiro',
            '4º': 'quarto', '5º': 'quinto', '6º': 'sexto',
            '7º': 'sétimo', '8º': 'oitavo', '9º': 'nono',
            '10º': 'décimo'
        }
        for num, word in number_map.items():
            text = text.replace(num, word)
        return text

    def normalize_dates(self, text: str) -> str:
        """Normaliza datas no texto para um formato padrão."""
        date_patterns = [
            (r'(\d{2})/(\d{2})/(\d{4})', '%d/%m/%Y'),
            (r'(\d{2})-(\d{2})-(\d{4})', '%d-%m-%Y'),
            (r'(\d{4})/(\d{2})/(\d{2})', '%Y/%m/%d')
        ]
        
        for pattern, date_format in date_patterns:
            matches = re.finditer(pattern, text)
            for match in matches:
                try:
                    date = datetime.strptime(match.group(), date_format)
                    month = MONTHS_PT[date.month]
                    normalized_date = f"{date.day} de {month} de {date.year}"
                    text = text.replace(match.group(), normalized_date)
                except ValueError:
                    continue
        return text

    def normalize_currency(self, text: str) -> str:
        """Normaliza valores monetários no texto."""
        currency_pattern = r'R\$\s*(\d+(?:\.\d{3})*(?:,\d{2})?)'
        matches = re.finditer(currency_pattern, text)
        for match in matches:
            value = match.group(1).replace('.', '').replace(',', '.')
            normalized = f"R$ {float(value):,.2f}".replace(',', 'X').replace('.', ',').replace('X', '.')
            text = text.replace(match.group(), normalized)
        return text

    def remove_typos(self, text: str) -> str:
        """Corrige erros comuns de digitação."""
        typo_map = {
            'vc': 'você',
            'pq': 'porque',
            'q': 'que',
            'tb': 'também',
            'tbm': 'também',
            'mt': 'muito',
            'mto': 'muito',
            'td': 'tudo',
            'tdo': 'tudo',
            'tda': 'toda',
            'tde': 'todo'
        }
        
        # Divide o texto em palavras e processa cada uma individualmente
        words = text.split()
        corrected_words = []
        for word in words:
            # Remove pontuação no final da palavra para fazer a comparação
            clean_word = word.strip('.,!?:;')
            # Verifica se a palavra está no mapa de correções
            if clean_word.lower() in typo_map:
                # Se houver pontuação, mantém ela no final
                punctuation = word[len(clean_word):]
                corrected = typo_map[clean_word.lower()] + punctuation
                corrected_words.append(corrected)
            else:
                corrected_words.append(word)
        
        return ' '.join(corrected_words)

    def remove_duplicate_text(self, text: str) -> str:
        """Remove texto duplicado mantendo apenas uma ocorrência."""
        words = text.split()
        seen = set()
        unique_words = []
        for word in words:
            if word not in seen:
                seen.add(word)
                unique_words.append(word)
        return ' '.join(unique_words)

    def normalize_abbreviations(self, text: str) -> str:
        """Normaliza abreviações comuns no texto."""
        abbr_map = {
            'ex.': 'exemplo',
            'etc.': 'etcetera',
            'vs.': 'versus',
            'i.e.': 'isto é',
            'e.g.': 'por exemplo',
            'Dr.': 'Doutor',
            'Dra.': 'Doutora',
            'Sr.': 'Senhor',
            'Sra.': 'Senhora',
            'Srta.': 'Senhorita'
        }
        for abbr, full in abbr_map.items():
            text = text.replace(abbr, full)
        return text

    def get_stemmer(self, language: str) -> SnowballStemmer:
        """Obtém ou cria um stemmer para o idioma especificado."""
        if language not in self.stemmers:
            try:
                self.stemmers[language] = SnowballStemmer(language)
            except ValueError:
                self.stemmers[language] = SnowballStemmer('english')
        return self.stemmers[language]

    def get_lemmatizer(self, language: str) -> WordNetLemmatizer:
        """Obtém ou cria um lemmatizer para o idioma especificado."""
        if language not in self.lemmatizers:
            self.lemmatizers[language] = WordNetLemmatizer()
        return self.lemmatizers[language]

    def stem_text(self, text: str, language: str = 'portuguese') -> str:
        """Aplica stemming ao texto."""
        stemmer = self.get_stemmer(language)
        words = text.split()
        stemmed_words = [stemmer.stem(word) for word in words]
        return ' '.join(stemmed_words)

    def lemmatize_text(self, text: str, language: str = 'portuguese') -> str:
        """Aplica lematização ao texto."""
        # Mapeamento de verbos em português para suas formas infinitivas
        verb_map = {
            'correndo': 'correr',
            'pulando': 'pular',
            'saltando': 'saltar',
            'fazendo': 'fazer',
            'dizendo': 'dizer',
            'vendo': 'ver',
            'indo': 'ir',
            'sendo': 'ser',
            'tendo': 'ter',
            'querendo': 'querer'
        }
        
        words = text.split()
        lemmatized_words = []
        for word in words:
            if word in verb_map:
                lemmatized_words.append(verb_map[word])
            else:
                lemmatized_words.append(word)
        return ' '.join(lemmatized_words)

    def clean_advanced(self, text: str, options: Dict[str, bool] = None) -> str:
        """
        Realiza uma limpeza avançada do texto com base nas opções especificadas.
        
        Opções disponíveis:
        - remove_emojis: Remove emojis e emoticons
        - remove_urls: Remove URLs
        - remove_emails: Remove endereços de email
        - normalize_numbers: Normaliza números por extenso
        - normalize_dates: Normaliza datas
        - normalize_currency: Normaliza valores monetários
        - remove_typos: Corrige erros comuns de digitação
        - remove_duplicates: Remove texto duplicado
        - normalize_abbreviations: Normaliza abreviações
        - stem: Aplica stemming
        - lemmatize: Aplica lematização
        """
        if options is None:
            options = {
                'remove_emojis': True,
                'remove_urls': True,
                'remove_emails': True,
                'normalize_numbers': True,
                'normalize_dates': True,
                'normalize_currency': True,
                'remove_typos': True,
                'remove_duplicates': True,
                'normalize_abbreviations': True,
                'stem': False,
                'lemmatize': False
            }

        if options.get('remove_emojis'):
            text = self.remove_emojis(text)
        if options.get('remove_urls'):
            text = self.remove_urls(text)
        if options.get('remove_emails'):
            text = self.remove_emails(text)
        if options.get('remove_typos'):
            text = self.remove_typos(text)
        if options.get('normalize_numbers'):
            text = self.normalize_numbers(text)
        if options.get('normalize_dates'):
            text = self.normalize_dates(text)
        if options.get('normalize_currency'):
            text = self.normalize_currency(text)
        if options.get('remove_duplicates'):
            text = self.remove_duplicate_text(text)
        if options.get('normalize_abbreviations'):
            text = self.normalize_abbreviations(text)

        language = self.detect_language(text)
        if options.get('stem'):
            text = self.stem_text(text, language)
        if options.get('lemmatize'):
            text = self.lemmatize_text(text, language)

        return text 


import pytest
from text_cleaner_for_py.advanced_cleaner import AdvancedTextCleaner

@pytest.fixture
def cleaner():
    return AdvancedTextCleaner()

def test_detect_language(cleaner):
    assert cleaner.detect_language("Olá, como vai você?") == "pt"
    assert cleaner.detect_language("Hello, how are you?") == "en"

def test_remove_emojis(cleaner):
    text = "Olá! 👋 Como vai você? 😊"
    assert cleaner.remove_emojis(text) == "Olá!  Como vai você? "

def test_remove_urls(cleaner):
    text = "Visite https://exemplo.com ou www.exemplo.org"
    assert cleaner.remove_urls(text) == "Visite  ou "

def test_remove_emails(cleaner):
    text = "Meu email é teste@exemplo.com"
    assert cleaner.remove_emails(text) == "Meu email é "

def test_normalize_numbers(cleaner):
    text = "1º lugar, 2º lugar, 3º lugar"
    assert cleaner.normalize_numbers(text) == "primeiro lugar, segundo lugar, terceiro lugar"

def test_normalize_dates(cleaner):
    text = "Data: 25/12/2023"
    assert "25 de dezembro de 2023" in cleaner.normalize_dates(text)

def test_normalize_currency(cleaner):
    text = "Preço: R$ 1.234,56"
    assert cleaner.normalize_currency(text) == "Preço: R$ 1.234,56"

def test_remove_typos(cleaner):
    text = "vc sabe pq isso aconteceu?"
    assert cleaner.remove_typos(text) == "você sabe porque isso aconteceu?"

def test_remove_duplicate_text(cleaner):
    text = "texto texto repetido repetido"
    assert cleaner.remove_duplicate_text(text) == "texto repetido"

def test_normalize_abbreviations(cleaner):
    text = "Dr. Silva e Dra. Santos"
    assert cleaner.normalize_abbreviations(text) == "Doutor Silva e Doutora Santos"

def test_stem_text(cleaner):
    text = "correndo pulando saltando"
    stemmed = cleaner.stem_text(text)
    assert "corr" in stemmed
    assert "pul" in stemmed
    assert "salt" in stemmed

def test_lemmatize_text(cleaner):
    text = "correndo pulando saltando"
    lemmatized = cleaner.lemmatize_text(text)
    assert "correr" in lemmatized
    assert "pular" in lemmatized
    assert "saltar" in lemmatized

def test_clean_advanced(cleaner):
    text = "Olá! 👋 Visite https://exemplo.com ou envie email para teste@exemplo.com. Data: 25/12/2023. Preço: R$ 1.234,56. vc sabe pq?"
    cleaned = cleaner.clean_advanced(text)
    assert "👋" not in cleaned
    assert "https://exemplo.com" not in cleaned
    assert "teste@exemplo.com" not in cleaned
    assert "25 de dezembro" in cleaned and "2023" in cleaned
    assert "R$ 1.234,56" in cleaned
    assert "você" in cleaned
    assert "porque" in cleaned

def test_clean_advanced_with_options(cleaner):
    text = "Olá! 👋 Visite https://exemplo.com"
    options = {
        'remove_emojis': True,
        'remove_urls': False,
        'remove_emails': True,
        'normalize_numbers': False,
        'normalize_dates': False,
        'normalize_currency': False,
        'remove_typos': False,
        'remove_duplicates': False,
        'normalize_abbreviations': False,
        'stem': False,
        'lemmatize': False
    }
    cleaned = cleaner.clean_advanced(text, options)
    assert "👋" not in cleaned
    assert "https://exemplo.com" in cleaned 


from text_cleaner_for_py.cleaner import (
    remove_accents,
    remove_special_characters,
    to_snake_case,
    to_camel_case,
    to_pascal_case,
    remove_html,
    remove_extra_spaces,
    clean_text,
)

from text_cleaner_for_py.cleaner_v1 import (
    normalize_text,
    remove_html_tags,
    clean_whitespace,
    filter_letters,
    filter_numbers,
    remove_stopwords,
)

from text_cleaner_for_py.advanced_cleaner import AdvancedTextCleaner

__version__ = "0.2.0"

__all__ = [
    # Funções básicas
    "remove_accents",
    "remove_special_characters",
    "to_snake_case",
    "to_camel_case",
    "to_pascal_case",
    "remove_html",
    "remove_extra_spaces",
    "clean_text",
    
    # Funções v1
    "normalize_text",
    "remove_html_tags",
    "clean_whitespace",
    "filter_letters",
    "filter_numbers",
    "remove_stopwords",
    
    # Classes avançadas
    "AdvancedTextCleaner",
]
